// Prevents additional console window on Windows in release, DO NOT REMOVE!!
#![cfg_attr(not(debug_assertions), windows_subsystem = "windows")]

use std::{collections::HashMap, sync::Mutex};

use config::Config;
use crossbeam::channel::Sender;
use overlay::{Event, LockedOverlayData, Overlay};

use poe_data::world_area::WorldArea;
use simple_logger::SimpleLogger;
use storage::Storage;
// use overlay::{Overlay, Manager};
use tauri::Manager;

mod config;
mod overlay;
mod plan;
mod storage;

#[tauri::command]
fn set_interactable(interactable: bool, state: tauri::State<Sender<Event>>) {
    log::info!("set_interactable: {interactable:?}");
    if interactable {
        state.send(overlay::State::Interactable {}.into()).ok();
    } else {
        state.send(overlay::Event::Return).ok();
    }
}

#[tauri::command]
fn set_auto_hide(auto_hide: bool, state: tauri::State<LockedOverlayData>) {
    log::info!("set_auto_hide: {auto_hide:?}");
    if let Ok(mut settings) = state.lock() {
        settings.auto_hide = auto_hide;
    }
}

#[tauri::command]
fn load_world_areas() -> HashMap<String, WorldArea> {
    log::info!("Loading world areas");
    poe_data::world_area::load_world_areas_map(include_str!(
        "../../data/processed_world_areas.json"
    ))
}

#[tauri::command]
fn load_config(state: tauri::State<Mutex<Storage>>) -> Config {
    if let Ok(storage) = state.lock() {
        log::info!("Loaded config from storage: {:?}", storage.config);
        return storage.config.clone();
    }
    panic!("ERROR!")
}

#[tauri::command]
fn set_config(config: Config, state: tauri::State<Mutex<Storage>>) {
    log::info!("Saved config: {:?}", config);
    if let Ok(mut storage) = state.lock() {
        storage.config = config;
        storage.save();
    }
}

fn main() {
    SimpleLogger::new()
        .with_module_level("underlayer", log::LevelFilter::Info)
        .with_level(log::LevelFilter::Trace)
        .init()
        .expect("Could not init logger");

    tauri::Builder::default()
        .setup(|app| {
            let tx = Overlay::initialize(
                app.get_window("Overlay")
                    .expect("Could not get main overlay window"),
                "Untitled 1 - Mousepad",
            );
            app.manage(tx);
            app.manage(Mutex::new(Storage::default()));


            app.get_window("Overlay")
                .expect("Could not get main overlay window")
                .open_devtools();

            Ok(())
        })
        .invoke_handler(tauri::generate_handler![
            set_interactable,
            set_auto_hide,
            load_world_areas,
            load_config,
            set_config,
        ])
        .run(tauri::generate_context!())
        .expect("error while running tauri application");
}
