use std::error::Error;

use directories::ProjectDirs;
use serde::{Deserialize, Serialize};

use crate::config::Config;
use crate::plan::Plan;

#[derive(Serialize, Deserialize, Debug)]
pub struct Storage {
    pub config: Config,
    #[serde(skip)]
    pub plan: Option<Plan>,
    pub plan_name: Option<String>,
}

const QUALIFIER: &'static str = "me";
const ORGANIZATION: &'static str = "isark.poe";
const APPLICATION: &'static str = "Nothing";
const CONFIG_FILE: &'static str = "configuration.json";
const SAVED_PLANS: &'static str = "plans";

fn proj_dir() -> Option<ProjectDirs> {
    ProjectDirs::from(QUALIFIER, ORGANIZATION, APPLICATION)
}

impl Default for Storage {
    fn default() -> Self {
        let config = Self::load();
        match config {
            Some(config) => config,
            None => Self {
                config: Default::default(),
                plan: Default::default(),
                plan_name: None,
            },
        }
    }
}

impl Storage {
    fn load() -> Option<Self> {
        let mut storage: Option<Storage> = serde_json::from_str(
            &std::fs::read_to_string(proj_dir()?.data_dir().with_file_name(CONFIG_FILE)).ok()?,
        )
        .ok();

        if let Some(storage) = &mut storage {
            if let Some(plan_name) = &storage.plan_name {
                storage.plan = Self::load_plan(plan_name);
            }
        }

        log::info!("Loaded storage: {:?}", storage);

        storage
    }

    fn load_plan<T: Into<String>>(file_name: T) -> Option<Plan> {
        serde_json::from_str(
            &std::fs::read_to_string(
                proj_dir()?
                    .data_dir()
                    .join(SAVED_PLANS)
                    .with_file_name(file_name.into()),
            )
            .ok()?,
        )
        .ok()
    }

    pub fn save(&self) {
        if let Ok(content) = serde_json::to_string_pretty(&self) {
            if let Some(dir) = proj_dir() {
                match std::fs::write(dir.data_dir().with_file_name(CONFIG_FILE), content) {
                    Ok(_) => {
                        if let Some(c) = dir.data_dir().with_file_name(CONFIG_FILE).to_str() {
                            log::info!("Saved config to {}", c)
                        }
                    }
                    Err(_) => log::error!("Could not write config"),
                }
            }
        }
    }
}
